import java.util.ArrayList;

import enumerate.Action;
import enumerate.State;
import fighting.Attack;


public class CharInfoPack {
	State state;
	Attack attack;
	Action action;
	HitBox box;
	Speed speed;
	Speed prevSpeed;
	int hp;
	int energy;
	int remainingFrames;
	
	static CharInfoPack neutral;
	static ArrayList<Action> actionsWitnessed = new ArrayList<Action>();
	static ArrayList<Integer> attacksWitnessed = new ArrayList<Integer>();
	static HitBox mem = null;
	
	public CharInfoPack(State state, Attack attack, Action action, HitBox box, Speed speed, Speed prev, int hp, int energy, int remainingFrames) {
		this.state = state;
		this.attack = attack;
		this.action = action;
		this.box = box;
		this.speed = speed;
		if(prev == null)
			this.prevSpeed = speed;
		else
			this.prevSpeed = prev;
		this.hp = hp;
		this.energy = energy;
		this.remainingFrames = remainingFrames;
		/* Old code for experimentation
		 * String s = action.toString();
		if(!actionsWitnessed.contains(action) && !s.contains("THROW") && (s.endsWith("A") || s.endsWith("B") || s.endsWith("C")))
		{
			actionsWitnessed.add(action);
			if(attack != null)
			{
				HitArea a = attack.getHitAreaNow();
				int dist = a.getR() - a.getL() + box.distance(a);
				attacksWitnessed.add(dist);
				//System.out.println("Action: "+action+": "+dist);
			}
			else
				mem = box;
		}
		else if(actionsWitnessed.size() > attacksWitnessed.size() && attack != null)
		{
			HitArea a = attack.getHitAreaNow();
			int dist = a.getR() - a.getL() + mem.distance(a);
			attacksWitnessed.add(dist);
			//System.out.println("Action: "+actionsWitnessed.get(actionsWitnessed.size()-1)+": "+dist);
		}*/
	}
	
	public CharInfoPack prediction(int frames)
	{
		State nstate;
		Attack nattack;
		Action naction;
		Speed nspeed;
		
		int xAccell = (speed.x - prevSpeed.x) * frames;
		int yAccell = (speed.y - prevSpeed.y) * frames;
		nspeed = new Speed(speed.x + xAccell, speed.y + yAccell);
		
		HitBox nbox = predict(box,nspeed,frames);
		
		if(onGround && state.equals(State.AIR))
			nstate = neutral.state;
		else
			nstate = state;
		
		if(frames >= remainingFrames)
			naction = neutral.action;
		else
			naction = action;
		
		if(attack == null)
			nattack = null;
		else
		{
			int predAttackFrame = attack.getNowFrame() + frames;
			if(predAttackFrame > attack.getActive())
				nattack = null;//new Attack();
			else
				nattack = attack;
		}
		
		return new CharInfoPack(nstate,nattack,naction,nbox,nspeed, speed, hp, energy, remainingFrames - frames);
	}
	
	boolean onGround;
	public HitBox predict(HitBox box, Speed speed, int frames)
	{
		
		HitBox pbox = box.getMovedBox(speed.x*frames,speed.y*frames);
		if(pbox.bottom > neutral.box.bottom) //lower bounded
			pbox = pbox.getMovedBox(0, neutral.box.bottom - pbox.bottom);
		
		onGround = pbox.bottom == neutral.box.bottom;
		
		return pbox;
	}
	
	public String toString()
	{
		if(attack == null)
			return "";
		return ""+attack.getActive();
	}
}
