import gameInterface.AIInterface;
import rule.GameStateInterface;
import rule.Rule;
import structs.CharacterData;
import structs.FrameData;
import structs.GameData;
import structs.Key;
import structs.MotionData;
import commandcenter.CommandCenter;

/**
 * Super class for dynamic scripting AIs.
 * 
 * @author Kevin Majchrzak
 * 
 */
public abstract class DynamicScriptingAI implements AIInterface,
		GameStateInterface {

	Key inputKey;
	boolean playerNumber;
	FrameData frameData;
	CommandCenter cc;
	GameData gd;
	MotionData myMotionData;
	MotionData enemyMotionData;
	CharacterData myCharacterData;
	CharacterData enemyCharacterData;
	Rule[] dynamicScript;

	@Override
	public void close() {

	}

	@Override
	public String getCharacter() {
		return CHARACTER_KFM;
	}

	@Override
	public void getInformation(FrameData frameData) {

		this.frameData = frameData;
		cc.setFrameData(frameData, playerNumber);
		if (!frameData.emptyFlag && frameData.remainingTime > 0) {
			myCharacterData = cc.getMyCharacter();
			enemyCharacterData = cc.getEnemyCharacter();
			// motion data
			if (playerNumber) {
				myMotionData = gd.getPlayerOneMotion().elementAt(
						myCharacterData.getAction().ordinal());
				enemyMotionData = gd.getPlayerTwoMotion().elementAt(
						enemyCharacterData.getAction().ordinal());
			} else {
				myMotionData = gd.getPlayerTwoMotion().elementAt(
						myCharacterData.getAction().ordinal());
				enemyMotionData = gd.getPlayerOneMotion().elementAt(
						enemyCharacterData.getAction().ordinal());
			}
		}
	}

	@Override
	public int initialize(GameData gameData, boolean playerNumber) {
		this.playerNumber = playerNumber;
		this.inputKey = new Key();
		cc = new CommandCenter();
		frameData = new FrameData();
		gd = gameData;

		initializeRuleBase();
		initializeDynamicScript();

		return 0;
	}

	public abstract void initializeRuleBase();

	public abstract void initializeDynamicScript();

	public abstract void updateDynamicScript();

	@Override
	public Key input() {
		return inputKey;
	}

	@Override
	public void processing() {

		if (!frameData.emptyFlag && frameData.getRemainingTime() > 0) {

			updateDynamicScript();

			if (cc.getskillFlag()) {
				inputKey = cc.getSkillKey();
			} else {
				inputKey.empty();
				// execute dynamic script
				for (Rule rule : dynamicScript) {
					if (rule.satisfied()) {
						rule.execute();
						break;
					}
				}
			}

		}

	}

	@Override
	public FrameData getFrameData() {
		return frameData;
	}

	@Override
	public boolean getPlayerNumber() {
		return playerNumber;
	}

	@Override
	public GameData getGameData() {
		return gd;
	}

	@Override
	public MotionData getMyMotionData() {
		return myMotionData;
	}

	@Override
	public MotionData getEnemyMotionData() {
		return enemyMotionData;
	}

	@Override
	public CharacterData getMyCharacterData() {
		return myCharacterData;
	}

	@Override
	public CharacterData getEnemyCharacterData() {
		return enemyCharacterData;
	}

	@Override
	public CommandCenter getCommandCenter() {
		return cc;
	}
}
