import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import enumerate.Action;


public class LearningControl {
	LittleFuzzy ai;
	
	private int howFarBackILook;
	static int thisGameUseThreshold;
	double similarityThreshold = 0.6;
	double battleFailThreshold = 0.2;
	
	ArrayList<Act> myActs = new ArrayList<Act>();
	ArrayList<Act> opActs = new ArrayList<Act>();
	
	ArrayList<RatedAct> myRatedActs = new ArrayList<RatedAct>();
	ArrayList<RatedAct> opRatedActs = new ArrayList<RatedAct>();
	ArrayList<RatedAct> allRatedActs = new ArrayList<RatedAct>();
	
	RatedAct lastChosen;
	int lastChosenIndex;
	
	protected String knowledgeFile = "knowledge.txt";
	
	/*
	 	Do I know a method that works? 
		If so, try it
		If not, do I know a method that hasn't been tested against this set of variables?
		If so, try it
		If not, try one of the methods with probability proportional to effectiveness
	 */
	
	public LearningControl(LittleFuzzy ai, int howFarBackILook, int thisGameUseThreshold)
	{
		this.ai = ai;
		this.howFarBackILook = howFarBackILook;
		LearningControl.thisGameUseThreshold = thisGameUseThreshold;
	}
	
	public RatedAct getBestAct()
	{
		Act act = new Act(ai.mePredicted, ai.opPredicted, null, null, 0, 0);
		
		//int success = 0;
		RatedAct successract = null;
		/*for(RatedAct ract : allRatedActs)
		{
			if(act.opAction == ract.opAction && act.myState == ract.myState && act.opState == ract.opState)
			{
				if(success < ract.success)
				{
					success = ract.success;
					successract = ract;
				}
				
			}
		}*/

		if(successract == null)
		{
			double similarity = similarityThreshold;
			for(int i=0; i<allRatedActs.size(); i++)
			{
				RatedAct ract = allRatedActs.get(i);
				if(ai.me.energy + ract.energyCost < 0)
					continue;
				if(ract.thisGameUses >= LearningControl.thisGameUseThreshold && ract.uses()/ract.successes() <= battleFailThreshold)
				{
					System.out.println("======== useless");
					continue;
				}
				double sim = act.similarScenario(ract);
				//System.out.print(ract.myAction+" "+sim+"; ");
				if(similarity < sim)
				{
					lastChosenIndex = i;
					similarity = sim;
					successract = ract;
					if(sim == 1)
						break;
				}
			}
			if(successract != null)
			{
				//System.out.print("Similarity: "+similarity+" ");
				//System.out.print("Dist: "+Math.abs(successract.dist - act.dist)+" ");
			}
		}
		
		lastChosen = successract;
		return successract;
	}
	
	public void addMyAct()
	{
		Act myAct = null;
		Act opAct = null;
		if(myActs.size() > 0)
			myAct = myActs.get(lastItemIn(myActs));
		if(opActs.size() > 0)
			opAct = opActs.get(lastItemIn(opActs));
		
		Act act = new Act(ai.me, ai.opponent, myAct, opAct, ai.me.box.right-ai.me.box.left, ai.me.energy-ai.myenergy);
		myActs.add(act);
		
		clean(myActs);
	}
	
	public void addOpAct()
	{
		Act myAct = null;
		Act opAct = null;
		if(myActs.size() > 0)
			myAct = myActs.get(lastItemIn(myActs));
		if(opActs.size() > 0)
			opAct = opActs.get(lastItemIn(opActs));
		
		Act act = new Act(ai.opponent, ai.me, opAct, myAct, ai.opponent.box.right-ai.opponent.box.left, ai.opponent.energy-ai.openergy);
		act.myCharacter = LittleFuzzy.OPCHAR;
		act.opCharacter = LittleFuzzy.MYCHAR;
		opActs.add(act);
		clean(opActs);
	}
	
	private void clean(ArrayList<Act> acts)
	{
		while(acts.size() > howFarBackILook)
		{
			acts.remove(0);
			Act update = acts.get(0);
			update.lastMyAct = null;
			update.lastOpAct = null;
		}
	}
	
	private int lastItemIn(Collection<?> c)
	{
		return c.size()-1;
	}
	
	public void whenHit(int damage)
	{
		Act aac = opActs.get(lastItemIn(opActs));
		RatedAct act = new RatedAct(aac,damage);
		
		for(RatedAct a : allRatedActs)
			if(a.similarAct(act) > 0.9)
			{
				a.successes += 1;
				a.thisGameSuccess += 1;
				System.out.println("Got hit");
				Collections.sort(allRatedActs);
				return;
			}
		
		opRatedActs.add(act);
		allRatedActs.add(act);
		Collections.sort(allRatedActs);
		
		if(ai.lessonMode)
		{
			ai.lessonAction = act.myAction;
			System.out.println("Move to learn: "+ai.lessonAction);
		}
		
		//System.out.println("Opponent hit");
		save();
	}
	
	public void whenHitting(int damage)
	{
		RatedAct act = new RatedAct(myActs.get(lastItemIn(myActs)),damage);
		
		for(RatedAct a : allRatedActs)
			if(a.similarAct(act) > 0.9)
			{
				a.successes += 1;
				a.thisGameSuccess += 1;
				//System.out.println("Hit!");
				return;
			}
		
		myRatedActs.add(act);
		allRatedActs.add(act);
		Collections.sort(allRatedActs);

		if(ai.lessonMode)
		{
			System.out.println("Move learnt: "+act.myAction+" vs "+act.opAction);
		}
		
		//System.out.println("I hit");
		save();
	}
	
	public void whenIUse(Action action)
	{
		RatedAct act = new RatedAct(myActs.get(lastItemIn(myActs)),0);
		
		for(RatedAct a : allRatedActs)
			if(a.similarAct(act) > 0.9)
			{
				a.uses++;
				a.thisGameUses++;
				if(a.hitboxWidth == 0)
					a.hitboxWidth = ai.me.box.right - ai.me.box.left;
				return;
			}
	}
	
	public void whenOpUses(Action action)
	{
		RatedAct act = new RatedAct(opActs.get(lastItemIn(opActs)),0);
		
		for(RatedAct a : allRatedActs)
			if(a.similarAct(act) > 0.9)
			{
				a.uses++;
				a.thisGameUses++;
				if(a.hitboxWidth == 0)
					a.hitboxWidth = ai.opponent.box.right - ai.opponent.box.left;
				return;
			}
	}
	
	public void save()
	{
		String out = "<version> 4\n";
		out += "<myacts>\n";
		
		for(RatedAct act : myRatedActs)
			out += act.toString()+"\n";
		
		out += "<opacts>\n";
		
		for(RatedAct act : opRatedActs)
			out += act.toString()+"\n";
		
		ai.write(knowledgeFile, out);
	}
	
	public void load()
	{
		String[] lines = ai.read(knowledgeFile);
		
		if(lines == null || lines.length < 2)
			return;
		
		boolean my = false;
		int version = 1;
		for(int i=0; i<lines.length; i++)
		{
			String line = lines[i];
			if(line.startsWith("<version>"))
				version = Integer.parseInt(line.substring(9).trim());
			else if(line.equals("<myacts>"))
				my = true;
			else if(line.equals("<opacts>"))
				my = false;
			else
			{
				RatedAct act = RatedAct.create(line,version);
				if(!my && version == 3)
				{
					String temp = act.myCharacter;
					act.myCharacter = act.opCharacter;
					act.opCharacter = temp;
				}
				act.priorKnowledge = true;
				if(my)
					myRatedActs.add(act);
				else
					opRatedActs.add(act);
				allRatedActs.add(act);
			}
		}
	}

}
















