package rl.simplerl;

import rule.Rule;

/**
 * Wrapper for rules with additional Information needed for the Reinforcement
 * Learning Algorithm.
 * 
 * @author Kevin Majchrzak
 * 
 */
public class RuleWrapper extends Rule {

	public long execCount;
	public Rule rule;
	public long weight;
	public boolean inScript;
	public boolean active;

	public RuleWrapper(Rule rule) {
		super(null);
		this.rule = rule;
		this.execCount = 0;
		this.weight = 0;
		inScript = false;
		this.isPermanentCombo = rule.isPermanentCombo;
		this.active = false;
	}

	@Override
	public boolean satisfied() {
		return rule.satisfied();
	}

	@Override
	public void execute() {
		execCount++;
		rule.execute();
	}

	@Override
	public int getPriority() {
		return rule.getPriority();
	}

	@Override
	public String toString() {
		return rule.getClass().getName();
	}

	@Override
	public int compareTo(Rule other) {
		if (other.getPriority() != this.getPriority()) {
			return super.compareTo(other);
		} else {
			// if same prio then sort by weight
			return (int) (((RuleWrapper) other).weight - this.weight);
		}
	}

}